/*
  EDMA IDFWizard GNOMIZED Version 0.2.0
  Copyright (C) 1998-1999 David Martnez Oliveira
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; see the file COPYING.  If not, write to
  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  Boston, MA 02111-1307, USA.
  
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <signal.h>
#include <time.h>
#include <unistd.h>
#include <gnome.h>

#include <edma.h>

/* App Forms */
#include "generalform.h"
#include "propform.h"
#include "metform.h"
#include "inhform.h"
#include "prefform.h"
#include "filemake.h"

/* Constants */
#define VERSION "0.2.2"
#define COPYRIGHT "(c) David Martnez Oliveira"

/* Function prtotypes */
int my_main(int argc,char *argv[]);
void HandleEx (int);
int ScriptTableInit();


/* Global vars */
OBJID     id2;
EUint32   crash;
CLASSID   cId=-1;
EChar     ProjectName[200];
EChar     CurrentDir[200];

GeneralWidgets ggw;
PropertyWidgets gpw;
MethodWidgets gmw;
InheritanceWidgets giw;
PrefWidgets gfw;

/*
** GNOME stuff
** 
*/
void about_callback (GtkWidget *widget, gpointer data)
{
  GtkWidget *aboutdlg = NULL;
  gchar *authors[] = {
    "David Martnez Oliviera",
    NULL
  };
  aboutdlg = gnome_about_new("GNOME EDMA IDFWizard", VERSION,
			     COPYRIGHT,
			     (const gchar **) authors,
			     "GNOME EDMA IDF Wizard allows you to genetare class interfaces and other required files for building classes. Works with GNU EDMA 0.7.1 or later",
			     NULL);
  gtk_widget_show(aboutdlg);
}

/* Callback function for main window delete event. Always allows delete. */
int delete_callback (GtkWidget *widget, gpointer data)
{
   return FALSE;
}

/* Callback function for main window destroy. Exits gtk_main_loop. */
void exit_callback (GtkWidget *widget, gpointer data)
{
    gtk_main_quit ();
}

void file_ok_sel (GtkWidget *w, GtkWidget *fs)
{
  gchar *fname;
  EChar hole[100];
  OBJID  parser;
  EChar  buf[500],name[200],type[200],access[200];
  EPChar aux,aux1;
  EUint32 i,ne,n,m_static,m_abstract;

  fname= gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
  g_print ("IDF FIle: %s\n", fname);
  /*  
  if (strstr(fname,".idf")==NULL) 
    {
      g_print ("No idf file");
      gtk_widget_destroy(fs);
      return;
    }
  if (strstr(fname,".IDF")==NULL) 
    {
      g_print ("No idf file");
      gtk_widget_destroy(fs);
      return;
    }
  */
  /* We have a IDF FIle */
  /* We create the IDF parser */
  parser=NewObj("EDMAIDF_PARSER",NULL);
  Met3(parser,"Open",fname);
  Met3(parser,"GetClassName",buf);
  g_print ("ClassName : %s\n",buf);
  GeneralSetClassName(&ggw,buf);

  Met3(parser,"GetModuleName",buf);
  g_print ("ClassName : %s\n",buf);
  GeneralSetModuleName(&ggw,buf);

  /* Adding property info */
  PropInit(&gpw);
  Met3(parser,"GetNProp",&n);
  g_print ("\nProcessing %d properties\n",n);
  for (i=0;i<n;i++)
    {
      Met3(parser,"GetPropName",i,name);
      Met3(parser,"GetPropType",i,type);
      Met3(parser,"GetPropAccess",i,access);
      Met3(parser,"GetPropNElems",i,&ne);
      PropSet(&gpw,name,type,access,ne);
    }

  /* Adding method info */
  MetInit(&gmw);
  Met3(parser,"GetNMet",&n);
  g_print ("\nProcessing %d properties\n",n);
  for (i=0;i<n;i++)
    {
      Met3(parser,"GetMetName",i,name);
      Met3(parser,"GetMetSignature",i,type);
      Met3(parser,"IsMetVirtual",i,&ne);
      Met3(parser,"IsMetStatic",i,&m_static);
      Met3(parser,"IsMetAbstract",i,&m_abstract);
      /* FIXME: We must to change EDMAIDF_PARSER*/
      MetSet(&gmw,name,type,ne,m_abstract,m_static);
    }

  /* Inheritance information */
  InhInit(&giw);
  memset (buf,0,500);
  Met3(parser,"GetSCList",buf);
  n=0;
  if (strlen(buf)!=0)
    {
      for (i=0;i<strlen(buf);i++)
	if (buf[i]==',')
	  n++;
      g_print ("Located %d superclasses",n+1);
      aux1=aux=buf;
      
      for (i=0;i<n;i++)
	{
	  aux=strchr(aux1,',');
	  *aux=0;
	  InhSet(&giw,aux1);
	  aux1=aux+1;
	  g_print ("SC:%s",aux1);
	}
      InhSet(&giw,aux1);
    }
  
  Met3(parser,"Close");
  g_print ("\nIDF File successfuly loaded");    
  FreeObj(parser);
  gtk_widget_destroy(fs);
}

void destroy (GtkWidget *widget, GtkWidget *fs)
{
  g_print ("\n Destroying File Selector");
  g_print ("\n");
  gtk_widget_destroy (widget);
}

/* Callback function for defining GNOME/GTK menus. */
void dummy_callback (GtkWidget *widget, gpointer data)
{
  
}

void new_callback (GtkWidget *widget, gpointer data)
{

  GeneralSetClassName(&ggw,"");
  GeneralSetModuleName(&ggw,"");
  PropInit(&gpw);
  MetInit(&gmw);
  InhInit(&giw);
}

void open_callback (GtkWidget *widget, gpointer data)
{
  GtkWidget *fs;
  
  
  fs=gtk_file_selection_new ("Open IDF File");

  /* Connect the ok_button to file_ok_sel function */
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fs)->ok_button),
		      "clicked", (GtkSignalFunc) file_ok_sel, fs );
  
  /* Connect the cancel_button to destroy the widget */
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (fs)->cancel_button),
			     "clicked", (GtkSignalFunc) destroy,
			     GTK_OBJECT (fs));
  gtk_widget_show(fs);
}

/* Menus Ops. */

void make_idf_file (GtkWidget *widget, gpointer data)
{
  GtkWidget *fs;
  EChar    temp[256];

  fs=gtk_file_selection_new ("Save IDF File");
  GeneralGetClassName (&ggw,temp);
  strcat (temp,".idf");
  gtk_file_selection_set_filename(GTK_FILE_SELECTION(fs),temp);
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fs)->ok_button),
		      "clicked", (GtkSignalFunc) save_idf_file, fs );
  
  /* Connect the cancel_button to destroy the widget */
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (fs)->cancel_button),
			     "clicked", (GtkSignalFunc) destroy,
			     GTK_OBJECT (fs));
  gtk_widget_show(fs);

}
void make_ines_file (GtkWidget *widget, gpointer data)
{
  GtkWidget *fs;
  EChar    temp[256];

  fs=gtk_file_selection_new ("Save INES File");
  GeneralGetClassName (&ggw,temp);
  strcat (temp,".ines");
  gtk_file_selection_set_filename(GTK_FILE_SELECTION(fs),temp);
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fs)->ok_button),
		      "clicked", (GtkSignalFunc) save_ines_file, fs );
  
  /* Connect the cancel_button to destroy the widget */
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (fs)->cancel_button),
			     "clicked", (GtkSignalFunc) destroy,
			     GTK_OBJECT (fs));
  gtk_widget_show(fs);


}

void make_c_file (GtkWidget *widget, gpointer data)
{
  GtkWidget *fs;
  EChar    temp[256];

  fs=gtk_file_selection_new ("Save INES File");
  GeneralGetClassName (&ggw,temp);
  strcat (temp,"_imp.c");
  gtk_file_selection_set_filename(GTK_FILE_SELECTION(fs),temp);
  /*
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fs)->ok_button),
		      "clicked", (GtkSignalFunc) save_c_file, fs );
  */
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fs)->ok_button),
		      "clicked", (GtkSignalFunc) save_imp_file, fs );


  /* Connect the cancel_button to destroy the widget */
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (fs)->cancel_button),
			     "clicked", (GtkSignalFunc) destroy,
			     GTK_OBJECT (fs));
  gtk_widget_show(fs);


}


void make_make_file (GtkWidget *widget, gpointer data)
{
  GtkWidget *fs;
  EChar    temp[256];

  fs=gtk_file_selection_new ("Save INES File");
  GeneralGetClassName (&ggw,temp);
  strcat (temp,".mk");
  gtk_file_selection_set_filename(GTK_FILE_SELECTION(fs),temp);
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fs)->ok_button),
		      "clicked", (GtkSignalFunc) save_make_file, fs );
  
  /* Connect the cancel_button to destroy the widget */
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (fs)->cancel_button),
			     "clicked", (GtkSignalFunc) destroy,
			     GTK_OBJECT (fs));
  gtk_widget_show(fs);


}



void make_doc_file (GtkWidget *widget, gpointer data)
{
  GtkWidget *fs;
  EChar    temp[256];

  fs=gtk_file_selection_new ("Save INES File");
  GeneralGetClassName (&ggw,temp);
  strcat (temp,".sgml");
  gtk_file_selection_set_filename(GTK_FILE_SELECTION(fs),temp);
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fs)->ok_button),
		      "clicked", (GtkSignalFunc) save_doc_file, fs );
  
  /* Connect the cancel_button to destroy the widget */
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (fs)->cancel_button),
			     "clicked", (GtkSignalFunc) destroy,
			     GTK_OBJECT (fs));
  gtk_widget_show(fs);


}

/* Builds Gnomish main menu in the application window (app) */
void create_main_menu (GtkWidget *app)
{
   GnomeUIInfo filemenu[] = {
      {GNOME_APP_UI_ITEM,
       N_("New"), N_("New Class Interface"),
       new_callback, NULL, NULL,
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_NEW,
       0, 0, NULL},
      {GNOME_APP_UI_ITEM,
       N_("Open"), N_("Open an IDF File"),
       open_callback, NULL, NULL,
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_OPEN,
       0, 0, NULL},
      {GNOME_APP_UI_ITEM,
       N_("Exit"), N_("Exit the application"),
       exit_callback, NULL, NULL,
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_QUIT,
       0, 0, NULL},
      GNOMEUIINFO_END
   };
   
   GnomeUIInfo makemenu[] = {
      {GNOME_APP_UI_ITEM,
       N_("Make Interface"), N_("Make Interface File"),
       make_idf_file, NULL, NULL,
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP,
       0, 0, NULL},
      {GNOME_APP_UI_ITEM,
       N_("Make Implementation Skel"), N_("Make Implementation File"),
       make_c_file, NULL, NULL,
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP,
       0, 0, NULL},
      {GNOME_APP_UI_ITEM,
       N_("Make Makefile Skel"), N_("Make Makefile"),
       make_make_file, NULL, NULL,
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP,
       0, 0, NULL},
      {GNOME_APP_UI_ITEM,
       N_("Make Doc Skel"), N_("Make Documentation Skel File"),
       make_doc_file, NULL, NULL,
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP,
       0, 0, NULL},
      {GNOME_APP_UI_ITEM,
       N_("Make INES"), N_("Make INES Installation spec File"),
       make_ines_file, NULL, NULL,
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP,
       0, 0, NULL},
      {GNOME_APP_UI_ITEM,
       N_("Make All"), N_("Make All the Files"),
       dummy_callback, NULL, NULL,
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP,
       0, 0, NULL},
      GNOMEUIINFO_END
   };
   
   
   GnomeUIInfo helpmenu[] = {
      {GNOME_APP_UI_ITEM, 
       N_("About..."), N_("Info about this program"),
       about_callback, NULL, NULL, 
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ABOUT,
       0, 0, NULL},
      GNOMEUIINFO_END
   };
   
   GnomeUIInfo mainmenu[] = {
      GNOMEUIINFO_SUBTREE(N_("File"), filemenu),
      GNOMEUIINFO_SUBTREE(N_("Make"), makemenu),
      //      GNOMEUIINFO_JUSTIFY_RIGHT,
      GNOMEUIINFO_SUBTREE(N_("Help"), helpmenu),
      GNOMEUIINFO_END
   };

   gnome_app_create_menus(GNOME_APP(app), mainmenu);
}


/* Helper functions */
void close_application( GtkWidget *widget, gpointer data )
{
       gtk_main_quit();
}


main(int argc,char *argv[])
{
 
  /* Initialize signal handler  */
  signal (SIGSEGV,HandleEx);
  signal (SIGTERM,HandleEx);
  signal (SIGQUIT,HandleEx);
  signal (SIGINT,HandleEx);
  
  /* Initialize EDMA System */
  /*EDMARTLink(); */
  EDMAInit();

  my_main(argc,argv);
  return 0;
}


int my_main(int argc,char *argv[])
{
  EUint32   i;
  GtkWidget *app;
  GtkWidget *note;
  GtkWidget *swindow,*swindow1,*swindow2;
  GtkWidget *window;
  GtkWidget *hpan;
  GtkWidget *vpan;
  GtkWidget *prop_box;
  GtkWidget *met_box;
  GtkWidget *class_list;
  GtkWidget *label1,*label2;
  GtkWidget *label,*edit,*combo,*aux;
  GList     *sos=NULL;
  GList     *lang=NULL;
  GList     *machines=NULL;
  GList     *proxies=NULL;
  GList     *parsers=NULL;
  GList     *types=NULL;
  GList     *acc=NULL;

  gnome_init("GIDFWizard",NULL,argc,argv);
  app=gnome_app_new ("GIDFWizard","EDMA IDFWizard Gnomized");
  gtk_widget_realize(app);
  
  /* Connect window signals */
  gtk_signal_connect (GTK_OBJECT (app), "delete_event",
		      GTK_SIGNAL_FUNC (delete_callback), NULL);
  gtk_signal_connect (GTK_OBJECT (app), "destroy",
		      GTK_SIGNAL_FUNC (exit_callback), NULL);  
  create_main_menu (app);
  
  window = gtk_vbox_new (FALSE,FALSE);   
  gnome_app_set_contents(GNOME_APP(app),window);
  note=gtk_notebook_new();
  
  gtk_container_add(GTK_CONTAINER(window),note);
  printf ("[DEBUG] Creating General Form\n");
  InitGeneralForm(&ggw);
  ggw.build_form(&ggw,note);
  printf ("[DEBUG] Creating Properties Form\n");
  InitPropForm(&gpw);
  gpw.build_form(&gpw,note);
  printf ("[DEBUG] Creating Methods Form\n");
  InitMetForm(&gmw);
  gmw.build_form(&gmw,note);
  printf ("[DEBUG] Creating Inheritance Form\n");
  InitInhForm(&giw);
  giw.build_form(&giw,note);
  printf ("[DEBUG] Creating Preferences Form\n");
  InitPrefForm(&gfw);
  gfw.build_form(&gfw,note);
  
  // Properties related controls
  
  gtk_notebook_set_page (GTK_NOTEBOOK(note), 0);
  
  gtk_widget_show(note);
  gtk_widget_show(window);
  gtk_widget_show(app);
  
  gtk_main ();
  printf ("\nAll Done!!");
  EDMAEnd();
  printf ("\nEDMA unloaded\n");
  return 0;
}

/* Exception handler */
void HandleEx (int s)
{
   if (crash)
     {
	printf ("\nException received while shutdown");
	exit(1);
     }
   else
     {
	crash=1;
	printf ("\n Signal recived: %d",s);
	printf ("\n%s",strsignal(s));
	printf ("\n EDMA abrupt shutdown required");
	EDMAEnd();
	exit (1);
     }
}










